from typing import Optional

import numpy as np

from _pydelatin import PydelatinTriangulator


class Delatin:
    def __init__(
        self,
        arr: np.ndarray,
        *,
        height: Optional[int] = None,
        width: Optional[int] = None,
        z_scale: float = 1,
        z_exag: float = 1,
        max_error: float = 0.001,
        max_triangles: Optional[int] = None,
        max_points: Optional[int] = None,
        base_height: float = 0,
        level: bool = False,
        invert: bool = False,
        blur: int = 0,
        gamma: float = 0,
        border_size: int = 0,
        border_height: float = 1
    ):
        """

        Args:
            - arr: data array. If a 2D array, dimensions are expected to be
              (height, width). If a 1D array, height and width parameters must
              be passed, and the array is assumed to be in C order.

        Kwargs:
            - height: height of array; required when arr is not 2D
            - width: width of array; required when arr is not 2D
            - z_scale: z scale relative to x & y
            - z_exag: z exaggeration
            - max_error: maximum triangulation error
            - max_triangles: maximum number of triangles
            - max_points: maximum number of vertices
            - base_height: solid base height
            - level: auto level input to full grayscale range
            - invert: invert heightmap
            - blur: gaussian blur sigma
            - gamma: gamma curve exponent
            - border_size: border size in pixels
            - border_height: border z height
        """
        max_triangles = max_triangles if max_triangles is not None else 0
        max_points = max_points if max_points is not None else 0

        if len(arr.shape) != 2:
            if height is None or width is None:
                msg = 'Height and width must be passed when arr is not 2D'
                raise ValueError(msg)
        else:
            height, width = arr.shape

        self.tri = PydelatinTriangulator(
            width,
            height,
            max_error,
            z_scale,
            z_exag,
            max_triangles,
            max_points,
            level,
            invert,
            blur,
            gamma,
            border_size,
            border_height,
            base_height,
        )
        self.tri.setData(arr.flatten())
        self.tri.run()

    @property
    def vertices(self):
        return self.tri.getPoints().reshape(-1, 3)

    @property
    def triangles(self):
        return self.tri.getTriangles().reshape(-1, 3).astype(np.uint32)

    @property
    def error(self):
        return self.tri.getError()



import math
from typing import Tuple

import numpy as np


# This is copied from pymartini
def decode_ele(png: np.ndarray, encoding: str) -> np.ndarray:
    """Decode array to elevations
    Arguments:
        - png (np.ndarray). Ndarray of elevations encoded in three channels,
          representing red, green, and blue. Must be of shape (tile_size,
          tile_size, >=3), where `tile_size` is usually 256 or 512
        - encoding: (str): Either 'mapbox' or 'terrarium', the two main RGB
          encodings for elevation values.
    Returns:
        (np.array) Array of shape (tile_size^2) with decoded elevation values
    """
    allowed_encodings = ['mapbox', 'terrarium']
    if encoding not in allowed_encodings:
        raise ValueError(f'encoding must be one of {allowed_encodings}')

    if png.shape[0] <= 4:
        png = png.T

    # Get bands
    if encoding == 'mapbox':
        red = png[:, :, 0] * (256 * 256)
        green = png[:, :, 1] * (256)
        blue = png[:, :, 2]

        # Compute float height
        terrain = (red + green + blue) / 10 - 10000
    elif encoding == 'terrarium':
        red = png[:, :, 0] * (256)
        green = png[:, :, 1]
        blue = png[:, :, 2] / 256

        # Compute float height
        terrain = (red + green + blue) - 32768

    return terrain


def rescale_positions(
    vertices: np.ndarray,
    bounds: Tuple[float, float, float, float],
    flip_y: bool = False,
):
    """Rescale positions to bounding box

    Args:
        - vertices: vertices output from Delatin
        - bounds: linearly rescale position values to this extent, expected to
          be [minx, miny, maxx, maxy].
        - flip_y: (bool) Flip y coordinates. Can be useful since images'
          coordinate origin is in the top left.

    Returns:
        (np.ndarray): ndarray of shape (-1, 3) with positions rescaled. Each row
        represents a single 3D point.
    """
    out = np.zeros(vertices.shape, dtype=np.float32)

    tile_size = vertices[:, :2].max()
    minx, miny, maxx, maxy = bounds
    x_scale = (maxx - minx) / tile_size
    y_scale = (maxy - miny) / tile_size

    if flip_y:
        scalar = np.array([x_scale, -y_scale])
        offset = np.array([minx, maxy])
    else:
        scalar = np.array([x_scale, y_scale])
        offset = np.array([minx, miny])

    # Rescale x, y positions
    out[:, :2] = vertices[:, :2] * scalar + offset
    out[:, 2] = vertices[:, 2]
    return out


def latitude_adjustment(lat: float):
    """Latitude adjustment for web-mercator projection

    Args:
        - lat: latitude in degrees
    """
    return math.cos(math.radians(lat))



# type: ignore
# pylint: disable=import-outside-toplevel
from glob import glob

from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import find_packages, setup

with open("README.md") as f:
    readme = f.read()

ext_modules = [
    Pybind11Extension(
        "_pydelatin",
        sorted(glob("src/*.cpp")),  # Sort source files for reproducibility
    ),
]


setup(
    name="pydelatin",
    version="0.2.7",
    python_requires=">=3.6",
    author="Kyle Barron",
    author_email="kylebarron2@gmail.com",
    url="https://github.com/kylebarron/pydelatin",
    description="A wrapper for hmm",
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(include=["pydelatin", "pydelatin.*"]),
    ext_modules=ext_modules,
    install_requires=["numpy"],
    extras_require={"test": ["pytest", "pytest-benchmark", "imageio"]},
    cmdclass={"build_ext": build_ext},
    zip_safe=False,
)
